@@ -11,8 +11,8 @@ from xml.etree import ElementTree
 from httpx import URL, AsyncClient
 from pydantic import BaseModel, validator
 
-from ._utils import ManyTasks, utcnow
-from .core import AwsClient
+from ._utils import ManyTasks, pretty_xml, utcnow
+from .core import AwsClient, RequestError
 
 if TYPE_CHECKING:
     from ._types import S3ConfigProtocol
@@ -32,6 +32,8 @@ class S3Config:
     aws_secret_key: str
     aws_region: str
     aws_s3_bucket: str
+    # custom host to connect with
+    aws_host: Optional[str] = None
 
 
 class S3File(BaseModel):
@@ -80,7 +82,7 @@ class S3Client:
             if (t := xml_root.find('NextContinuationToken')) is not None:
                 continuation_token = t.text
             else:
-                raise RuntimeError(f'unexpected response from S3: {r.text!r}')
+                raise RuntimeError(f'unexpected response from S3:\n{pretty_xml(r.content)}')
 
     async def delete(self, *files: Union[str, S3File]) -> List[str]:
         """
@@ -151,12 +153,25 @@ class S3Client:
         https://docs.aws.amazon.com/AmazonS3/latest/dev/RESTAuthentication.html#RESTAuthenticationQueryStringAuth
         """
         assert not path.startswith('/'), 'path should not start with /'
-        url = URL(f'https://{self._aws_client.host}/{path}')
+        url = URL(f'{self._aws_client.endpoint}/{path}')
         url = self._aws_client.add_signed_download_params('GET', url, max_age)
         if version:
             url = url.copy_add_param('v', version)
         return str(url)
 
+    async def download(self, file: Union[str, S3File], version: Optional[str] = None) -> bytes:
+        if isinstance(file, str):
+            path = file
+        else:
+            path = file.key
+
+        url = self.signed_download_url(path, version=version)
+        r = await self._aws_client.client.get(url)
+        if r.status_code == 200:
+            return r.content
+        else:
+            raise RequestError(r)
+
     def signed_upload_url(
         self,
         *,
@@ -201,7 +216,7 @@ class S3Client:
             'Policy': b64_policy,
             **self._aws_client.signed_upload_fields(now, b64_policy),
         }
-        return dict(url=f'https://{self._aws_client.host}/', fields=fields)
+        return dict(url=f'{self._aws_client.endpoint}/', fields=fields)
 
 
 def to_key(sf: Union[S3File, str]) -> str:
