@@ -16,7 +16,7 @@ logger = logging.getLogger('aioaws.sns')
 
 
 class SnsWebhookError(ValueError):
-    def __init__(self, message: str, details: Any = None, headers: Dict[str, str] = None):
+    def __init__(self, message: str, details: Any = None, headers: Optional[Dict[str, str]] = None):
         super().__init__(message)
         self.message = message
         self.details = details
@@ -60,7 +60,7 @@ async def verify_webhook(request_body: Union[str, bytes], http_client: AsyncClie
 
 async def verify_signature(payload: SnsPayload, http_client: AsyncClient) -> None:
     url = payload.signing_cert_url
-    if not re.fullmatch(r'sns\.[a-z0-9\-]+\.amazonaws\.com', url.host):
+    if url.host is None or not re.fullmatch(r'sns\.[a-z0-9\-]+\.amazonaws\.com', url.host):
         raise SnsWebhookError(f'invalid SigningCertURL "{url}"')
 
     certs_content = await get_resources(url, http_client)
